/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef CFILE_H
#define CFILE_H

#include <stdio.h>
#include <string.h>
#include <cstdio>

#include "IStreamDaq.h"

/**
 *\file CFile.h
 *\date  Created on: 22-apr-2009
 *\author  Author: Claudio Salvadori
 *\brief  Implementation of IStream interface to read a file. 
*	This class reads a file and creates packet shaped 
*         like the SDef structure directives
 */

 /**
 *\class CFile
  *\brief  Implementation of IStream interface to read a file. 
*	This class reads a file and creates packet shaped 
*         like the SDef structure directives
 */
class CFile: implements IStreamDaq{
	public:
		/**\brief CFile destructor*/		
		~CFile();

		/**\brief Initialize the stream reader (
		*\param def 	Pointer to a stream definition structure (input)
		*\param sty		Style of reading (by length, or by start and end character)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret getDefinition (struct SDef def, s_read_style sty);
		
		/**\brief Blocking method to wait the data to be passed to formatter 
		*	  (maybe after a decode and a "divide in message" processes 
		*	  respectively to interpret the escape charactera and to divide
		*	  packets to the correct length).
		*\param buffer 	Pointer to a pointer of a vector of byte to be formatted (output)
		*\param length 	Pointer to the packet lenght variable (output)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret retStream(unsigned char **buffer, unsigned long *length);
	private:
		/**\brief This function execute the handshaking to initialize the stream
		*\param  style 	Handshacking style
		*\return 	s_ret value about sending from \see s_ret
		*/
		s_ret File_handshaking(handshake_style style);

		char*			f_Name;			/**< File path */
		file_type		f_File_Type;		/**< File type */
		s_read_style		f_Style;		/**< Reading style*/
		unsigned long		f_Size;			/**< Packet total size */
		unsigned char		f_Start;		/**< Start character */
		unsigned char		f_End;			/**< End character */
		FILE*			fdn;			/**< File descriptor number*/	
};
#endif
