/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CFORMATTER_H
#define CFORMATTER_H

#include <string.h>

#include "IFormatter.h"

/**
 * \file CFormatter.h
 *\date  Created on: 22-apr-2009
 *\author  Author: Claudio Salvadori
 *\brief  Class to format data received from stream
*	 based on the metadata vector (created by the xml parser) \see SMData
 */
/**
* \class CFormatter
  *\brief  Class to format data received from stream
*	 based on the metadata vector (created by the xml parser) \see SMData
 */
class CFormatter: implements IFormatter{

	public:
		
		/**\brief Constructor
		*\param header 	Pointer to header string vector 
		*\param data 	Pointer to formatted data vector 
		*/
		CFormatter(vector<SData>* data);

		/**\brief Destructor*/
		~CFormatter();

		/**\brief This method creates the header vector and the formatted data vector 
		*\param metadata	Metadata vector
		*\return One of the possible value of f_ret \see f_ret  
		*/
		virtual f_ret init (vector<struct SMData>* metadata);
		
		/**\brief Fill the vector of formatted data 
		*\param msg 	Pointer to a char vector of data to be formatted received from IStreamReader (input) 
		*\return One of the possible value of f_ret \see f_ret */
		virtual f_ret getData (unsigned char* pkt);

	private:
		/**\brief This method creates the data structure for the data vector, read by the metadata vector
		*\return One of the possible value of f_ret \see f_ret */
		f_ret createDataV ();
		
		/**\brief Method to format integer values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting integer vector (input)
		*\return This method returns the pointer of the integer vector*/
		int* getInt32(unsigned char *v, unsigned long _size);

		/**\brief Method to format unsigned integer values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting unsigned integer vector (input)
		*\return This method returns the pointer of the unsigned integer vector*/
		unsigned int* getUInt32(unsigned char *v, unsigned long _size);

		/**\brief Method to format char values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting char vector (input)
		*\return This method returns the pointer of the char vector*/
		char* getInt8(unsigned char *v, unsigned long _size);	

		/**\brief Method to format unsigned char values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting unsigned char vector (input)
		*\return This method returns the pointer of the unsigned char vector*/
		unsigned char* getUInt8(unsigned char *v, unsigned long _size);	
		
		/**\brief Method to format short values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting short vector (input)
		*\return This method returns the pointer of the char vector*/
		short* getInt16(unsigned char *v, unsigned long _size);	

		/**\brief Method to format unsigned short values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting unsigned short vector (input)
		*\return This method returns the pointer of the unsigned char vector*/
		unsigned short* getUInt16(unsigned char *v, unsigned long _size);	

		/**\brief Method to format long values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting long vector (input)
		*\return This method returns the pointer of the long vector*/
		long* getInt64(unsigned char *v, unsigned long _size);

		/**\brief Method to format unsigned long values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting unsigned long vector (input)
		*\return This method returns the pointer of the unsigned long vector*/
		unsigned long* getUInt64(unsigned char *v, unsigned long _size);

		/**\brief Method to format float values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting float vector (input)
		*\return This method returns the pointer of the float vector*/
		float* getFloat32(unsigned char *v, unsigned long _size);

		/**\brief Method to format double values
		*\param v	Pointer to a subset of stream data (input)
		*\param _size	Size of the resulting double vector (input)
		*\return This method returns the pointer of the double#include <stdio.h> vector*/
		double* getFloat64(unsigned char *v, unsigned long _size);


		vector<SData>* 			f_Data;		/**<SData vector \see SData */
		vector<struct SMData>* 		f_Metadata;	/**<Metadata vector \see SMData */

};
#endif
