/*
Copyright (C) 2009 - Paolo Pagano
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file DAQSData.cxx
 *\author Paolo Pagano
 *\brief  DAQSData functions development
 */

/**
 *\file DAQSData.cpp
 *\author Paolo Pagano
 *\brief  DAQSData functions development (installed version) 
 */

#include "TRandom.h"
#include "TDirectory.h"
#include "TProcessID.h"
#include "stdio.h"
#include <iostream>

#include "DAQEvent.h"

using namespace std;


void initRenderer(void);
void renderImage(unsigned char *image, unsigned int w, unsigned int h, unsigned int channel);

ClassImp(DAQSData)

DAQSData::DAQSData(const DAQSData &orig) : TObject(orig){
#ifdef DAQIMAGE
	daqp.width_ = orig.daqp.width_;
	daqp.height_ = orig.daqp.height_;
	daqp.duration_ = orig.daqp.duration_;
	daqp.lost_ = orig.daqp.lost_;
	daqp.ymean_ = orig.daqp.ymean_;
	memcpy ((UChar_t*)orig.daqp.image_, (UChar_t*)daqp.image_, IMAGE_BUFFER);
#endif

#ifdef DAQSOCKET
	for(int i = 0; i < MESSAGE_BUFFER; i++)
		daqp.message_[i] = orig.daqp.message_[i];

	daqp.counter_ = orig.daqp.counter_;
#endif

#ifdef DAQTIMERAVR
	daqp.type_[0] = orig.daqp.type_[0];
	daqp.type_[1] = orig.daqp.type_[1];
	daqp.type_[2] = orig.daqp.type_[2];

	daqp.ncycles_[0] = orig.daqp.ncycles_[0];
	daqp.ncycles_[1] = orig.daqp.ncycles_[1];
	daqp.ncycles_[2] = orig.daqp.ncycles_[2];

	daqp.ts_[0] = orig.daqp.ts_[0];
	daqp.ts_[1] = orig.daqp.ts_[1];
	daqp.ts_[2] = orig.daqp.ts_[2];

	daqp.seqNum_[0] = orig.daqp.seqNum_[0];
	daqp.seqNum_[1] = orig.daqp.seqNum_[1];
	daqp.seqNum_[2] = orig.daqp.seqNum_[2];

	daqp.bcnNum_[0] = orig.daqp.bcnNum_[0];
	daqp.bcnNum_[1] = orig.daqp.bcnNum_[1];	
	daqp.bcnNum_[2] = orig.daqp.bcnNum_[2];
#endif

#ifdef DAQTIMERUW_EE
	daqp.time_ = orig.daqp.time_;
	daqp.length_ = orig.daqp.length_;
#endif

#ifdef DAQTIMERUW
	daqp.time_ = orig.daqp.time_;
	daqp.length_ = orig.daqp.length_;
#endif
}
//______________________________________________________________________________
DAQSData::DAQSData() : TObject(){
#ifdef DAQIMAGE
	daqp.width_ = 0;	
	daqp.height_ = 0;
	daqp.duration_ = 0;
	daqp.lost_ = 0;
	daqp.ymean_ = 0;
	memset(daqp.image_, '\0', IMAGE_BUFFER);
#endif

#ifdef DAQSOCKET
	for(int i = 0; i < MESSAGE_BUFFER; i++)
		daqp.message_[i] = 0;

	daqp.counter_ = 0;
#endif

#ifdef DAQTIMERAVR
	daqp.type_[0] = 0;
	daqp.type_[1] = 0;
	daqp.type_[2] = 0;

	daqp.ncycles_[0] = 0;
	daqp.ncycles_[1] = 0;
	daqp.ncycles_[2] = 0;

	daqp.ts_[0] = 0;
	daqp.ts_[1] = 0;
	daqp.ts_[2] = 0;

	daqp.seqNum_[0] = 0;
	daqp.seqNum_[1] = 0;
	daqp.seqNum_[2] = 0;

	daqp.bcnNum_[0] = 0;
	daqp.bcnNum_[1] = 0;	
	daqp.bcnNum_[2] = 0;
#endif

#ifdef DAQTIMERUW_EE
	daqp.time_ = 0;
	daqp.length_ = 0;
#endif

#ifdef DAQTIMERUW
	daqp.time_ = 0;
	daqp.length_ = 0;
#endif
}
//______________________________________________________________________________

DAQSData::DAQSData(std::vector<SData> sdata_) : TObject(){
#ifdef DAQIMAGE
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Width"){
			//fprintf(stderr, "%s", sdata_[i].d_Name.c_str());
			daqp.width_ = *((UShort_t*)(sdata_[i].d_Data));
		}else if(sdata_[i].d_Name == "Height"){
			//fprintf(stderr, "%s", sdata_[i].d_Name.c_str());
			daqp.height_ = *(UShort_t*)(sdata_[i].d_Data);
		}else if(sdata_[i].d_Name == "Duration"){
			//fprintf(stderr, "%s", sdata_[i].d_Name.c_str());
			daqp.duration_ = *(UInt_t*)(sdata_[i].d_Data);
		}else if(sdata_[i].d_Name == "Lost"){
			//fprintf(stderr, "%s", sdata_[i].d_Name.c_str());
			daqp.lost_ = *(UShort_t*)(sdata_[i].d_Data);
		}else if(sdata_[i].d_Name == "YMean"){
			//fprintf(stderr, "%s", sdata_[i].d_Name.c_str());
			daqp.ymean_ = *(UShort_t*)(sdata_[i].d_Data);
		}else if(sdata_[i].d_Name == "Image"){
			//fprintf(stderr, "%s: ", sdata_[i].d_Name.c_str());
			memcpy(daqp.image_, (UChar_t*)sdata_[i].d_Data, IMAGE_BUFFER);	
			initRenderer();
			renderImage((unsigned char*) sdata_[i].d_Data, 160, 120, 1);	
		}
	}
#endif

#ifdef DAQSOCKET
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Message"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			for(unsigned int j = 0; j < MESSAGE_BUFFER - 1; j++)
				daqp.message_[j] = *((Char_t*)(sdata_[i].d_Data) + j);
			daqp.message_[MESSAGE_BUFFER - 1] = (Char_t)'\0';
			printf("%s\n ", (char*) daqp.message_);	
		}else if(sdata_[i].d_Name == "Counter"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.counter_ = *(UInt_t*)(sdata_[i].d_Data);
			printf("%u\n ", daqp.counter_);			
		}
	}
#endif

#ifdef DAQTIMERAVR	
		for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Cycles_1"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.ncycles_[0] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.ncycles_[0]);	
		}else if(sdata_[i].d_Name == "Cycles_2"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.ncycles_[1] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.ncycles_[1]);			
		}else if(sdata_[i].d_Name == "Cycles_3"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.ncycles_[2] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.ncycles_[2]);
		}else if(sdata_[i].d_Name == "TN_1"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.ts_[0] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.ts_[0]);
		}else if(sdata_[i].d_Name == "TN_2"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.ts_[1] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.ts_[1]);
		}else if(sdata_[i].d_Name == "TN_3"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.ts_[2] = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.ts_[2]);
		}else if(sdata_[i].d_Name == "SeqNum_1"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.seqNum_[0] = *(UChar_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.seqNum_[0]);
		}else if(sdata_[i].d_Name == "SeqNum_2"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.seqNum_[1] = *(UChar_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.seqNum_[1]);
		}else if(sdata_[i].d_Name == "SeqNum_3"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.seqNum_[2] = *(UChar_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.seqNum_[2]);
		}else if(sdata_[i].d_Name == "BcnNum_1"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.bcnNum_[0] = *(UChar_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.bcnNum_[0]);
		}else if(sdata_[i].d_Name == "BcnNum_2"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.bcnNum_[1] = *(UChar_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.bcnNum_[1]);
		}else if(sdata_[i].d_Name == "BcnNum_3"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.bcnNum_[2] = *(UChar_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.bcnNum_[2]);
		}
	}
#endif

#ifdef DAQTIMERUW_EE
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Time"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.time_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.time_);	
		}else if(sdata_[i].d_Name == "Length"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.length_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.length_);			
		}
	}
#endif

#ifdef DAQTIMERUW
	for(unsigned int i = 0; i < sdata_.size(); i++){
		if(sdata_[i].d_Name == "Time"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.time_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.time_);	
		}else if(sdata_[i].d_Name == "Length"){
			printf("%s: ", sdata_[i].d_Name.c_str());
			daqp.length_ = *(UShort_t*)(sdata_[i].d_Data);
			printf("%u, \n ", daqp.length_);			
		}
	}
#endif
}
//______________________________________________________________________________
void DAQSData::printData() const{
#ifdef DAQIMAGE
	printf("DAQSData::printData():");
	cout << "Width: " << daqp.width_ << endl;
	cout << "Height: " << daqp.height_ << endl;
	cout << "Duration: " << daqp.duration_ << endl;
	cout << "Lost: " << daqp.lost_ << endl;
	cout << "YMean: " << daqp.ymean_ << endl;
	cout << "Image: " << endl;
	for (int k =0; k < IMAGE_BUFFER; k++){
		printf("0x%x ", daqp.image_[k]);
		if (k % 10 == 0) printf("\n");
	}
#endif

#ifdef DAQSOCKET
	printf("Message: ");
	for(unsigned int i = 0; i < MESSAGE_BUFFER - 1; i++)
		printf("%c, ",  *(((char*)daqp.message_) + i));
	printf("\n");
	printf("Counter: %u\n",  daqp.counter_);
#endif

#ifdef DAQTIMERAVR
	printf("Type[0]: %u\n",  daqp.type_[0]);
	printf("Type[1]: %u\n",  daqp.type_[1]);
	printf("Type[2]: %u\n",  daqp.type_[2]);

	printf("NCycles[0]: %u\n",  daqp.ncycles_[0]);
	printf("NCycles[1]: %u\n",  daqp.ncycles_[1]);
	printf("NCycles[2]: %u\n",  daqp.ncycles_[2]);

	printf("TS[0]: %u\n",  daqp.ts_[0]);
	printf("TS[1]: %u\n",  daqp.ts_[1]);
	printf("TS[2]: %u\n",  daqp.ts_[2]);

	printf("SeqNum[0]: %u\n",  daqp.seqNum_[0]);
	printf("SeqNum[1]: %u\n",  daqp.seqNum_[1]);
	printf("SeqNum[2]: %u\n",  daqp.seqNum_[2]);

	printf("BcnNum[0]: %u\n",  daqp.bcnNum_[0]);
	printf("BcnNum[1]: %u\n",  daqp.bcnNum_[1]);
	printf("BcnNum[2]: %u\n",  daqp.bcnNum_[2]);
#endif

#ifdef DAQTIMERUW_EE
	printf("Time: %u\n",  daqp.time_);
	printf("Packet length: %u\n",  daqp.length_);
#endif

#ifdef DAQTIMERUW
	printf("Time: %u\n",  daqp.time_);
	printf("Packet length: %u\n",  daqp.length_);
#endif
}
//______________________________________________________________________________
void DAQSData::Clear(Option_t * /*option*/)
{
}


